@@ -40,11 +40,11 @@ openstack catalog list
 
 export GNOCCHI_SERVICE_TOKEN=$(openstack token issue -c id -f value)
 export GNOCCHI_SERVICE_URL=$(openstack catalog show metric -c endpoints -f value | awk '/public/{print $2}')
+export GNOCCHI_AUTHORIZATION=""  # Temporary set to transition to the new functional testing
 
 curl -X GET ${GNOCCHI_SERVICE_URL}/v1/archive_policy -H "Content-Type: application/json"
 
-# NOTE(sileht): gabbi tests needs assert on some Ceilometer resource types
-sudo gnocchi-upgrade --create-legacy-resource-types
+sudo gnocchi-upgrade
 
 # Just ensure tools still works
 gnocchi metric create
@@ -57,7 +57,7 @@ sudo chown -R tempest:stack $BASE/data/tempest
 # Run tests with tempst
 cd $BASE/new/tempest
 set +e
-sudo -H -u tempest OS_TEST_TIMEOUT=$TEMPEST_OS_TEST_TIMEOUT tox -eall-plugin -- --concurrency=$TEMPEST_CONCURRENCY gnocchi
+sudo -H -u tempest OS_TEST_TIMEOUT=$TEMPEST_OS_TEST_TIMEOUT tox -eall-plugin -- gnocchi --concurrency=$TEMPEST_CONCURRENCY
 TEMPEST_EXIT_CODE=$?
 set -e
 if [[ $TEMPEST_EXIT_CODE != 0 ]]; then
