@@ -21,7 +21,6 @@ import uuid
 
 import fixtures
 from oslotest import base
-from oslotest import mockpatch
 import six
 from six.moves.urllib.parse import unquote
 try:
@@ -136,6 +135,18 @@ class FakeRadosModule(object):
             self._ensure_key_exists(key)
             self.kvs[key] = value
 
+        def write(self, key, value, offset):
+            self._validate_key(key)
+            try:
+                current = self.kvs[key]
+            except KeyError:
+                current = b""
+            if len(current) < offset:
+                current += b'\x00' * (offset - len(current))
+            self.kvs[key] = (
+                current[:offset] + value + current[offset + len(value):]
+            )
+
         def stat(self, key):
             self._validate_key(key)
             if key not in self.kvs:
@@ -367,12 +378,50 @@ class TestCase(base.BaseTestCase):
     ARCHIVE_POLICIES = {
         'no_granularity_match': archive_policy.ArchivePolicy(
             "no_granularity_match",
-            0,
-            [
+            0, [
                 # 2 second resolution for a day
                 archive_policy.ArchivePolicyItem(
                     granularity=2, points=3600 * 24),
-                ],
+            ],
+        ),
+        'low': archive_policy.ArchivePolicy(
+            "low", 0, [
+                # 5 minutes resolution for an hour
+                archive_policy.ArchivePolicyItem(
+                    granularity=300, points=12),
+                # 1 hour resolution for a day
+                archive_policy.ArchivePolicyItem(
+                    granularity=3600, points=24),
+                # 1 day resolution for a month
+                archive_policy.ArchivePolicyItem(
+                    granularity=3600 * 24, points=30),
+            ],
+        ),
+        'medium': archive_policy.ArchivePolicy(
+            "medium", 0, [
+                # 1 minute resolution for an day
+                archive_policy.ArchivePolicyItem(
+                    granularity=60, points=60 * 24),
+                # 1 hour resolution for a week
+                archive_policy.ArchivePolicyItem(
+                    granularity=3600, points=7 * 24),
+                # 1 day resolution for a year
+                archive_policy.ArchivePolicyItem(
+                    granularity=3600 * 24, points=365),
+            ],
+        ),
+        'high': archive_policy.ArchivePolicy(
+            "high", 0, [
+                # 1 second resolution for an hour
+                archive_policy.ArchivePolicyItem(
+                    granularity=1, points=3600),
+                # 1 minute resolution for a week
+                archive_policy.ArchivePolicyItem(
+                    granularity=60, points=60 * 24 * 7),
+                # 1 hour resolution for a year
+                archive_policy.ArchivePolicyItem(
+                    granularity=3600, points=365 * 24),
+            ],
         ),
     }
 
@@ -416,7 +465,6 @@ class TestCase(base.BaseTestCase):
         self.coord.stop()
 
         self.archive_policies = self.ARCHIVE_POLICIES.copy()
-        self.archive_policies.update(archive_policy.DEFAULT_ARCHIVE_POLICIES)
         for name, ap in six.iteritems(self.archive_policies):
             # Create basic archive policies
             try:
@@ -426,18 +474,18 @@ class TestCase(base.BaseTestCase):
 
         self.conf.set_override(
             'driver',
-            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "null"),
+            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file"),
             'storage')
 
     def setUp(self):
         super(TestCase, self).setUp()
         if swexc:
-            self.useFixture(mockpatch.Patch(
+            self.useFixture(fixtures.MockPatch(
                 'swiftclient.client.Connection',
                 FakeSwiftClient))
 
-        self.useFixture(mockpatch.Patch('gnocchi.storage.ceph.rados',
-                                        FakeRadosModule()))
+        self.useFixture(fixtures.MockPatch('gnocchi.storage.ceph.rados',
+                                           FakeRadosModule()))
 
         if self.conf.storage.driver == 'file':
             tempdir = self.useFixture(fixtures.TempDir())
